#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2024, Anaconda, Inc., and Bokeh Contributors.
# All rights reserved.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
''' Provide Bokeh model "building block" classes.

One of the central design principals of Bokeh is that, regardless of
how the plot creation code is spelled in Python (or other languages),
the result is an object graph that encompasses all the visual and
data aspects of the scene. Furthermore, this *scene graph* is to be
serialized, and it is this serialized graph that the client library
BokehJS uses to render the plot. The low-level objects that comprise
a Bokeh scene graph are called :ref:`Models <bokeh.model>`.

'''

#-----------------------------------------------------------------------------
# Boilerplate
#-----------------------------------------------------------------------------
from __future__ import annotations

import logging # isort:skip
log = logging.getLogger(__name__)

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

# Bokeh imports
from ..model import Model
from . import (
    annotations,
    axes,
    callbacks,
    canvas,
    coordinates,
    css,
    expressions,
    filters,
    formatters,
    glyphs,
    graphs,
    grids,
    labeling,
    layouts,
    map_plots,
    mappers,
    plots,
    ranges,
    renderers,
    scales,
    selections,
    selectors,
    sources,
    text,
    textures,
    tickers,
    tiles,
    tools,
    transforms,
    ui,
    widgets,
)
from .annotations import *
from .axes import *
from .callbacks import *
from .canvas import *
from .coordinates import *
from .css import *
from .expressions import *
from .filters import *
from .formatters import *
from .glyphs import *
from .graphs import *
from .grids import *
from .labeling import *
from .layouts import *
from .map_plots import *
from .mappers import *
from .plots import *
from .ranges import *
from .renderers import *
from .scales import *
from .selections import *
from .selectors import *
from .sources import *
from .text import *
from .textures import *
from .tickers import *
from .tiles import *
from .tools import *
from .transforms import *
from .ui import *
from .widgets import *

#-----------------------------------------------------------------------------
# Globals and constants
#-----------------------------------------------------------------------------

__all__ = (
    "Model",
    *annotations.__all__,
    *axes.__all__,
    *callbacks.__all__,
    *canvas.__all__,
    *coordinates.__all__,
    *css.__all__,
    *expressions.__all__,
    *filters.__all__,
    *formatters.__all__,
    *glyphs.__all__,
    *graphs.__all__,
    *grids.__all__,
    *labeling.__all__,
    *layouts.__all__,
    *map_plots.__all__,
    *mappers.__all__,
    *plots.__all__,
    *ranges.__all__,
    *renderers.__all__,
    *scales.__all__,
    *selections.__all__,
    *selectors.__all__,
    *sources.__all__,
    *text.__all__,
    *textures.__all__,
    *tickers.__all__,
    *tiles.__all__,
    *tools.__all__,
    *transforms.__all__,
    *ui.__all__,
    *widgets.__all__,
)

#-----------------------------------------------------------------------------
# General API
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Dev API
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Private API
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Code
#-----------------------------------------------------------------------------
