#-----------------------------------------------------------------------------
# Copyright (c) 2012 - 2024, Anaconda, Inc., and Bokeh Contributors.
# All rights reserved.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------

json = {
    "attrs": {
        "Axis": {
            "major_tick_in": 0,
            "major_tick_out": 3,
            "major_tick_line_alpha": 0.25,
            "major_tick_line_color": "#5B5B5B",

            "minor_tick_line_alpha": 0.25,
            "minor_tick_line_color": "#5B5B5B",

            "axis_line_alpha": 1,
            "axis_line_color": "#5B5B5B",

            "major_label_text_color": "#5B5B5B",
            "major_label_text_font": "Calibri Light",
            "major_label_text_font_size": "0.95em",
            "major_label_text_font_style": "bold",

            "axis_label_standoff": 10,
            "axis_label_text_color": "#5B5B5B",
            "axis_label_text_font": "Calibri Light",
            "axis_label_text_font_size": "1.15em",
            "axis_label_text_font_style": "bold",
        },

        "Legend": {
            "spacing": 8,
            "glyph_width": 15,

            "label_standoff": 8,
            "label_text_color": "#5B5B5B",
            "label_text_font": "Calibri Light",
            "label_text_font_size": "0.95em",
            "label_text_font_style": "bold",

            "border_line_alpha": 0,
            "background_fill_alpha": 0.25,
        },

        "BaseColorBar": {
            "title_text_color": "#5B5B5B",
            "title_text_font": "Calibri Light",
            "title_text_font_size": "1.15em",
            "title_text_font_style": "bold",

            "major_label_text_color": "#5B5B5B",
            "major_label_text_font": "Calibri Light",
            "major_label_text_font_size": "0.95em",
            "major_label_text_font_style": "bold",

            "major_tick_line_alpha": 0,
            "bar_line_alpha": 0,
        },

        "Title": {
            "text_color": "#5B5B5B",
            "text_font": "Calibri Light",
            "text_font_size": "1.25em",
            "text_font_style": "bold",
        },
    },
}
