from typing import Any

def publish_display_data(data: dict[str, Any], metadata: dict[Any, Any] | None = ...,
    source: str | None = ..., *, transient: dict[str, Any] | None = ..., **kwargs: Any) -> None: ...
